namespace BrokenLink
{
   using System;
   using System.Globalization;
   using System.Windows;
   using System.Windows.Data;

   internal class ConnectionPointRelativeToAbsoluteValueConverter : IMultiValueConverter
   {
      /// <inheritdoc/>
      public object Convert(object[] values, Type targetType, object parameter, CultureInfo culture)
      {
         if (values == null || values.Length < 5 || !(values[0] is Point) || !(values[1] is double) || !(values[2] is double) || !(values[3] is double) || !(values[4] is double))
         {
            return values;
         }

         var location = (Point)values[0]; // Percentage Value.
         var controlWidth = (double)values[1];
         var controlHeight = (double)values[2];
         var connectionPointWidth = (double)values[3];
         var connectionPointHeight = (double)values[4];

         var left = ((location.X / 100) * controlWidth) - (connectionPointWidth / 2);
         var top = ((location.Y / 100) * controlHeight) - (connectionPointHeight / 2);

         left = Math.Min(Math.Max(0, left), controlWidth - connectionPointWidth);
         top = Math.Min(Math.Max(0, top), controlHeight - connectionPointHeight);

         return new Thickness(left, top, 0, 0);
      }

      /// <inheritdoc/>
      public object[] ConvertBack(object value, Type[] targetTypes, object parameter, CultureInfo culture)
      {
         throw new NotImplementedException();
      }
   }
}